************************************************************************
*
*  program to convert a CHG or a XDATCAR file 
*  to a chg.dx (data-explorer file)
*
************************************************************************

      PROGRAM DRAWCH       
        
C-----------------------------------------------------------------------
C PARAMETER:
C ----------
C   NRXD, NRYD, NRZD ... how often copy supercell in x y z
C   NIOND ... max number of ions
C   NEIGD ... max number of nearest neighbours
C   NGXD, NGYD, NGZD ... max number of NGXF, NGYF, NGZF (fine FFT-mesh)
C   NGZMV, NGYMV, NGXMV ... move-vector
C-----------------------------------------------------------------------

C-----------------------------------------------------------------------
C Ideen: Dynamische Arrays (f90!)
C        Controllfile einlesen (falls vorhanden)
C        NEIGD checken !!!
C-----------------------------------------------------------------------

      PARAMETER (NRXD=1,NRYD=1,NRZD=1)
      INTEGER  :: NRX=NRXD,NRY=NRYD,NRZ=NRZD
      PARAMETER (NIOND=160,NEIGD=20)
      PARAMETER (NGXD=128,NGYD=128,NGZD=160)
      PARAMETER (NPLWVD=NGXD*NRXD*NGYD*NRYD*NGZD*NRZD)
      PARAMETER (NC=MAX(NGXD,NGYD,NGZD))
      PARAMETER (NIONDT=NIOND*NRXD*NRYD*NRZD)

      DIMENSION C(NGXD*NRXD,NGYD*NRYD,NGZD*NRZD),POS(3,NIONDT)
      DIMENSION NEIGT(NIONDT,NEIGD)
      DIMENSION NEIGN(NIONDT),ANEIG(NIONDT)
      DIMENSION NEIGNL(NIONDT),NEIGTL(NIONDT,NEIGD)
      DIMENSION TMP(NC)
      DIMENSION A(3,3),B(3,3),ANORM(3),BNORM(3)  
      CHARACTER*40 SZ1,SZ2,SIONS
      LOGICAL LCHRG,LDIFF,LEIN

C---!!!!!! NPSPTS=10000 for LDIFF=.TRUE.
      PARAMETER (NPSPTS=1,NTYPD=2)
      DIMENSION PSPRHO(NPSPTS,NTYPD),ZVAL(NTYPD),PSGMAX(NTYPD),
     &          NITYP(NTYPD)
      DIMENSION LPCTX(NGXD),LPCTY(NGYD),LPCTZ(NGZD)
      COMPLEX   CHTOT(NPLWVD),CHTOTR(NPLWVD),CWORK(NPLWVD)
      DIMENSION NGPTAR(3)
      COMPLEX   CSTRF(NPLWVD,NTYPD)
      COMPLEX   CITPI
      COMMON /CONPI/    CITPI,TPI,PI
      PARAMETER (AMIX=0.2,TETSOL=109)

C-----------------------------------------------------------------------
C LDIFF=.TRUE. -> calculate difference between charge and atomic charge
C-----------------------------------------------------------------------
      LDIFF=.FALSE.
C-----------------------------------------------------------------------
C some constants and initialisation
C-----------------------------------------------------------------------
      PI   = 3.141 592 653 589 793 238
      TPI  = PI*2
      CITPI= CMPLX(0.,TPI)

      NGXMV=0 ; NGYMV=0 ; NGZMV=0      

C-----------------------------------------------------------------------
C  read POTCAR
C-----------------------------------------------------------------------
      CALL READAT(NPSPTS,NTYPD,NTYP,ZVAL,PSGMAX,PSPRHO)
 	
C--------------------------------------------------------------------
C  open CHG-file or XDATCAR file
C--------------------------------------------------------------------
      OPEN(10,FILE='CHG',STATUS='OLD',ERR=1201)
      LCHRG=.TRUE.
      WRITE(*,*)'Reading from CHG file'
      GOTO 1210
 1201 OPEN(10,FILE='ELFCAR',STATUS='OLD',ERR=1200)
      LCHRG=.TRUE.
      WRITE(*,*)'Reading from ELFCAR file'
      GOTO 1210
 1200 OPEN(10,FILE='XDATCAR',STATUS='OLD')
      WRITE(*,*)'Reading from XDATCAR file'
      WRITE(*,*)'(I think this part is not very good)'
      LCHRG=.FALSE.
	
 1210 OPEN(20,FILE='chg.dx')
      OPEN(21,FILE='pos.dx')

C ISET ... number of first dataset (for connecting .dx files)
C ISKIP ... if XDATCAR -> every ISKIP position
C           if CHG or ELFCAR -> only each ISKIP charge-value
      WRITE(*,*)'start set value (ISET), and skip (ISKIP) ?'
      READ(*,*) ISET,ISKIP

      ISET0=ISET

C=======================================================================
C  read CHG or ELFCAR  (new file format)
C=======================================================================
 1300 WRITE(*,*)'nr.:',ISET
      
      IF (LCHRG) THEN 
        READ(10,'(A40)',END=2000) SZ1
        READ(10,*) SCALE
        DO 1000 I=1,3
          READ(10,*) A(1,I),A(2,I),A(3,I)
          A(1,I)=SCALE*A(1,I)
          A(2,I)=SCALE*A(2,I)
          A(3,I)=SCALE*A(3,I)    
 1000   CONTINUE                     

        READ(10,'(A)') SIONS
C---if the POTCAR-reader is working correct, than one has to check
C---here if the NTYP is equal to the POTCAR-file
        NTYP=0
        DO
          READ(SIONS,*,ERR=1001,END=1001) (N,I=0,NTYP)
          NTYP=NTYP+1
        ENDDO
 1001   BACKSPACE 10
        IF (NTYP.GT.NTYPD) THEN
          WRITE(*,*) 'too many atom-types',NTYP,NTYPD
          STOP
        ENDIF
        READ(10,*) (NITYP(NT),NT=1,NTYP)
        NIONS=0
        DO NT=1,NTYP
          NIONS=NIONS+NITYP(NT)
        ENDDO
        WRITE(*,*)NTYP,'different ion types'
        WRITE(*,*)NIONS,'ions found on file'
        IF(NIONS.GT.NIOND) THEN
          WRITE(*,*) 'ERROR: Check NIOND'
          STOP
        ENDIF

        READ(10,*) 

        DO 1010 K=1,NIONS
          READ(10,*) POS(1,K),POS(2,K),POS(3,K)
C          IF (POS(3,K).GT.0.85) POS(3,K)=POS(3,K)-1.
C          IF (POS(2,K).GT.0.75) POS(2,K)=POS(2,K)-1.
C          IF ((POS(1,K).GT.0.90).AND.(K.NE.30)) POS(1,K)=POS(1,K)-1.
 1010   CONTINUE  

C-----------------------------------------------------------------------
C  read in charge-density
C-----------------------------------------------------------------------
        READ(10,*) NGX,NGY,NGZ
        WRITE(*,*) NGX,NGY,NGZ,'grid points found'
        IF(NGX.GT.NGXD.OR.NGY.GT.NGYD.OR.NGZ.GT.NGZD) THEN
          WRITE(*,*) 'ERROR: Check NGXD, NGYD, NGZD'
          STOP
        ENDIF
    
        READ(10,'(10F8.3)') (((C(N1,N2,N3),N1=1,NGX),N2=1,NGY),N3=1,NGZ)
        WRITE(*,*)'charge read in'

C-----------------------------------------------------------------------
C  duplicate charge-density
C-----------------------------------------------------------------------
        NRX=NGXD/NGX ; NRY=NGYD/NGY ; NRZ=NGZD/NGZ
        WRITE(*,*) 'duplication NRX NRY NRZ - maximum:',NRX,NRY,NRZ
        READ(*,*) I,II,III
        IF ((I.GE.1).OR.(I.LE.NRX)) NRX=I
        IF ((II.GE.1).OR.(II.LE.NRY)) NRY=II
        IF ((III.GE.1).OR.(III.LE.NRZ)) NRZ=III

        IF(NIONS*NRX*NRY*NRZ.GT.NIONDT) THEN
          WRITE(*,*) 'ERROR: Check NIONDT'
          STOP
        ENDIF

        FNRX=FLOAT(NRX)
        FNRY=FLOAT(NRY)
        FNRZ=FLOAT(NRZ)
        WRITE(*,*) 'NRX=',NRX,'NRY=',NRY,'NRZ=',NRZ
        IF(NRX.GT.1.OR.NRY.GT.1.OR.NRZ.GT.1) THEN
          DO 1050 KX=0,NRX-1
          DO 1050 KY=0,NRY-1
          DO 1050 KZ=0,NRZ-1
            IF(KX.NE.0.OR.KY.NE.0.OR.KZ.NE.0) THEN
              DO 1051 IX=1,NGX
              DO 1051 IY=1,NGY
              DO 1051 IZ=1,NGZ
 1051           C(KX*NGX+IX,KY*NGY+IY,KZ*NGZ+IZ)=C(IX,IY,IZ)
            ENDIF
 1050     CONTINUE
          NGX=NRX*NGX
          NGY=NRY*NGY
          NGZ=NRZ*NGZ
        ENDIF

C-----------------------------------------------------------------------
C LEIN=.TRUE -> draw Einheitszelle
C-----------------------------------------------------------------------
        WRITE(*,*) 'draw simple cell (T/F)?'
        READ(*,'(L1)')  LEIN

C=======================================================================
C  read XDATCAR-file
C=======================================================================
      ELSE
        IF (ISET.EQ.ISET0) THEN 
          NRX=1 ; NRY=1 ; NRZ=1
          FNRX=FLOAT(NRX)
          FNRY=FLOAT(NRY)
          FNRZ=FLOAT(NRZ)
          NGX=100 ; NGY=100 ; NGZ=100

          READ(10,*) NIONS,NATA,NNF
          IF(NIONS.GT.NIOND) THEN
            WRITE(*,*) 'ERROR: Check NIOND'
            STOP
          ENDIF

          READ(10,*) VOL,SIZEX,SIZEY,SIZEZ,TIME
          READ(10,*) TEMP
          READ(10,'(A40)') SZ1  
          READ(10,'(A40)') SZ2  
          WRITE(*,*) TYPO,ATART
          SIZEX=SIZEX*1E10
          SIZEY=SIZEY*1E10
          SIZEZ=SIZEZ*1E10
          SCALE=SIZEX
          TIME =TIME*1E12
 
C          A(1,1)=1.06066*3.5314
C          A(2,1)=-.6123724*3.5314
C          A(3,1)=0
C          A(1,2)=1.06066*3.5314
C          A(2,2)=.6123724*3.5314
C          A(3,2)=0
C          A(1,3)=0
C          A(2,3)=0
C          A(3,3)=6.49519*3.5314
          A=0
          A(1,1)=SIZEX
          A(2,2)=SIZEY
          A(3,3)=SIZEZ
          IREAD=0
          NGX=100
          NGY=100
          NGZ=100
        ELSE
          NIONS=NIONS/NRX/NRY/NRZ+ISK
        ENDIF

 1120   READ(10,*,END=2000)
        ISK=0
        DO K=1,ISK
          READ(10,*) RDUM,RDUM,RDUM
        ENDDO
        NIONS=NIONS-ISK
        DO 1110 K=1,NIONS
          READ(10,*) POS(1,K),POS(2,K),POS(3,K)
 1110   CONTINUE
        IREAD=IREAD+1

        IF (MOD(IREAD-1,ISKIP).NE.0) GOTO 1120
      ENDIF

C-----------------------------------------------------------------------
C now duplicate the positions
C-----------------------------------------------------------------------
      DO 1011 I=0,NRX-1
      DO 1011 II=0,NRY-1
      DO 1011 III=0,NRZ-1
      DO 1011 K=1,NIONS
        IIII=K+(I+NRX*(II+NRY*III))*NIONS
        POS(1,IIII)=POS(1,K)+I
        POS(2,IIII)=POS(2,K)+II
        POS(3,IIII)=POS(3,K)+III
 1011 CONTINUE
      NIONS=NIONS*NRX*NRY*NRZ

C-----------------------------------------------------------------------
C  ask user for cutoff's
C-----------------------------------------------------------------------
      CALL LATTIC(OMEGA,A,B,ANORM,BNORM,6)

      IF (ISET.EQ.ISET0) THEN 
        WRITE(*,*)'nearest neighboar cutoff (DNEIG1,DNEIG2) ?' 
        READ(*,*) DNEIG1,DNEIG2
        IF(DNEIG2.LE.(DNEIG1+.01)) THEN
          DNEIG2=DNEIG1+.01
          WRITE(*,*)'DNEIG2 should be at least .01 greater than DNEIG1'
          WRITE(*,*)'I have changed DNEIG2 to DNEIG1 + .01 for you'
        ENDIF
        WRITE(*,*)'cutoff for atoms on bounderies?'
        READ(*,*) DMINE
        WRITE(*,*)'cutoff for charge on bounderies?'
        READ(*,*) DMIN
        WRITE(*,*)'witdh  for cutoff-function?'
        READ(*,*) WI               
        WRITE(*,*)"shifts NGXMV,NGYMV,NGZMV"
        READ(*,*) NGXMV,NGYMV,NGZMV
        AMAX=50.0
        DO 240 NI=1,NIONS
	  ANEIG(NI)=AMAX
  240   CONTINUE
      ENDIF

C-----------------------------------------------------------------------
C  move  in x y z   ... I think this should be done before the NN-stuff
C-----------------------------------------------------------------------
      IF ((NGZMV.NE.0).OR.(NGYMV.NE.0).OR.(NGXMV.NE.0)) THEN
        IF (LCHRG) THEN
          DO 500 NX=1,NGX
          DO 500 NY=1,NGY
            DO 510 NZ=1,NGZ
  510         TMP(NZ)=C(NX,NY,MOD(NZ-1+NGZMV+NGZ,NGZ)+1)
            DO 520 NZ=1,NGZ
  520         C(NX,NY,NZ)=TMP(NZ)
  500     CONTINUE

          DO 600 NZ=1,NGZ
          DO 600 NX=1,NGX
            DO 610 NY=1,NGY
  610         TMP(NY)=C(NX,MOD(NY-1+NGYMV+NGY,NGY)+1,NZ)
            DO 620 NY=1,NGY
  620         C(NX,NY,NZ)=TMP(NY)
  600     CONTINUE

          DO 700 NZ=1,NGZ
          DO 700 NY=1,NGY
            DO 710 NX=1,NGX
  710         TMP(NX)=C(MOD(NX-1+NGXMV+NGX,NGX)+1,NY,NZ)
            DO 720 NX=1,NGX
  720         C(NX,NY,NZ)=TMP(NX)
  700     CONTINUE
        ENDIF
 
        RX=FNRX/NGX
        RY=FNRY/NGY
        RZ=FNRZ/NGZ
C        WRITE(*,*) FNRX,FNRY,FNRZ
C        WRITE(*,*) NGX,NGY,NGZ
C        WRITE(*,*) RX,RY,RZ
        DO 800 NI=1,NIONS
C          WRITE(*,*) (POS(I,NI),I=1,3)
          POS(1,NI)=MOD(POS(1,NI)-NGXMV*RX+50.0,FNRX)
          POS(2,NI)=MOD(POS(2,NI)-NGYMV*RY+50.0,FNRY)
          POS(3,NI)=MOD(POS(3,NI)-NGZMV*RZ+50.0,FNRZ)
C          WRITE(*,*) (POS(I,NI),I=1,3)
  800   CONTINUE
      ENDIF

C-----------------------------------------------------------------------
C build up nearest neighbour table
C-----------------------------------------------------------------------
      NIONSS=NIONS/NRX/NRY/NRZ
      DO 250 NI=1,NIONS
        IND  =0
        INDL =0
        AKOOR=0
        DO 260 NII=1,NIONS

C  periodic boundary conditions
          DX = MOD(POS(1,NI)-POS(1,NII)+50.5,FNRX) -.5
          DY = MOD(POS(2,NI)-POS(2,NII)+50.5,FNRY) -.5
          DZ = MOD(POS(3,NI)-POS(3,NII)+50.5,FNRZ) -.5
          D  = SQRT((DX*A(1,1)+DY*A(1,2)+DZ*A(1,3))**2
     &            + (DX*A(2,1)+DY*A(2,2)+DZ*A(2,3))**2
     &            + (DX*A(3,1)+DY*A(3,2)+DZ*A(3,3))**2)
          IF (NII.NE.NI.AND.D.LT.DNEIG2) THEN
            INDL=INDL+1
            IF (INDL.GT.NEIGD) THEN
              WRITE(*,*) 'ERROR: Check NEIGD'
              STOP
            ENDIF
            NEIGTL(NI,INDL)=NII
            IF (D.LT.DNEIG1) THEN 
              AKOOR=AKOOR +1
            ELSE
              AKOOR=AKOOR + (DNEIG2-D)/(DNEIG2-DNEIG1)
            ENDIF
          ENDIF

C  no periodic boundary conditions
          DX = POS(1,NI)-POS(1,NII)
          DY = POS(2,NI)-POS(2,NII)
          DZ = POS(3,NI)-POS(3,NII)
          D  = SQRT((DX*A(1,1)+DY*A(1,2)+DZ*A(1,3))**2
     &            + (DX*A(2,1)+DY*A(2,2)+DZ*A(2,3))**2
     &            + (DX*A(3,1)+DY*A(3,2)+DZ*A(3,3))**2)
          IF (NII.NE.NI.AND.D.LT.DNEIG2) THEN
C next lines for me - don't draw HH-Bonds less than 1.29 
C            IF (MOD(NII-1,NIONSS).LT.(NIONSS-6).AND.
C     &          MOD(NI-1 ,NIONSS).LT.(NIONSS-6)) THEN
              IND=IND+1
              IF (IND.GT.NEIGD) THEN
                WRITE(*,*) 'ERROR: Check NEIGD'
                STOP
              ENDIF
              NEIGT(NI,IND)=NII
C              WRITE(*,*) NI,NII,D
C            ELSE IF (D.LT.1.29) THEN
C              IND=IND+1
C              NEIGT(NI,IND)=NII
C            ENDIF
          ENDIF 
 260    CONTINUE
C       ANEIG(NI) =AKOOR
        NEIGN(NI) =IND
        NEIGNL(NI)=INDL
 250  CONTINUE

C-----------------------------------------------------------------------
C average bond angle
C-----------------------------------------------------------------------
      DO 270 N1=1,NIONS
        TET =0
        TET2=0
        TETM=0
        NTET=0
        NEI=NEIGNL(N1)
        DO 280 I =1  ,NEI
        DO 280 II=I+1,NEI
          N2=NEIGTL(N1,I)
          N3=NEIGTL(N1,II)
          D1 = MOD(POS(1,N1)-POS(1,N2)+50.5,FNRX) -.5
          D2 = MOD(POS(2,N1)-POS(2,N2)+50.5,FNRY) -.5
          D3 = MOD(POS(3,N1)-POS(3,N2)+50.5,FNRZ) -.5
          DX1 = D1*A(1,1)+D2*A(1,2)+D3*A(1,3)
          DY1 = D1*A(2,1)+D2*A(2,2)+D3*A(2,3)
          DZ1 = D1*A(3,1)+D2*A(3,2)+D3*A(3,3)
          AD1=   SQRT(DX1*DX1+DY1*DY1+DZ1*DZ1)

          D1 = MOD(POS(1,N1)-POS(1,N3)+50.5,FNRX) -.5
          D2 = MOD(POS(2,N1)-POS(2,N3)+50.5,FNRY) -.5
          D3 = MOD(POS(3,N1)-POS(3,N3)+50.5,FNRZ) -.5
          DX2 = D1*A(1,1)+D2*A(1,2)+D3*A(1,3)
          DY2 = D1*A(2,1)+D2*A(2,2)+D3*A(2,3)
          DZ2 = D1*A(3,1)+D2*A(3,2)+D3*A(3,3)
          AD2=   SQRT(DX2*DX2+DY2*DY2+DZ2*DZ2)

          THETA=ACOS((DX1*DX2+DY1*DY2+DZ1*DZ2)/AD1/AD2)/PI*180

          TET = TET +THETA
          TET2= TET2+THETA**2
          NTET= NTET+1
          TETM= TETM+(TETSOL-THETA)**2

  280   CONTINUE
        TETP =TET /NTET
        TET2P=TET2/NTET
        TETM =TETM/NTET 
        IF (NTET.GE.2)  THEN 
          ANEIG(N1)=ANEIG(N1)*(1-AMIX)+MIN(SQRT(TETM),AMAX)*AMIX
        ELSE
          ANEIG(N1)=AMAX
        ENDIF
  270 CONTINUE

      F1=1./NGX
      F2=1./NGY
      F3=1./NGZ 

C-----------------------------------------------------------------------
C  search for ions near edges of cube
C-----------------------------------------------------------------------
C      IF (.FALSE.) THEN
      IF (LCHRG) THEN 
        WRITE(*,*)'starting work on charge'
        DMINC=DMIN/SCALE

        IF (LDIFF) THEN 
          CALL INILPC (NGX,NGY,NGZ,LPCTX,LPCTY,LPCTZ)
          NGPTAR(1)=NGX
          NGPTAR(2)=NGX
          NGPTAR(3)=NGX

          NITYP(1)=NIONS
          NPLWV=NGX*NGY*NGZ
          CALL STUFAK(NGX,NGY,NGZ,NIONS,NTYP,NITYP,NPLWV,
     &                POS,CSTRF,LPCTX,LPCTY,LPCTZ)

          CALL RHOATO(.TRUE.,NGX,NGY,NGZ,LPCTX,LPCTY,LPCTZ,NPLWV,
     &                NTYP,NITYP,CSTRF,B,
     &                PSGMAX,NPSPTS,PSPRHO,CHTOT,CHTOTR,NGPTAR,CWORK)
 
          K=1
          DO 410 N3=1,NGZ
          DO 410 N2=1,NGY
          DO 410 N1=1,NGX
            C(N1,N2,N3)=C(N1,N2,N3)-CHTOTR(K)
            K=K+1
  410     CONTINUE

        ELSE IF (DMINC.NE.0) THEN 
          WRITE(*,*)DMINC
          DO 300 NI=1,NIONS
            IF (POS(1,NI).LT.DMINC .OR. POS(1,NI) .GT. NRX-DMINC  .OR.
     &          POS(2,NI).LT.DMINC .OR. POS(2,NI) .GT. NRY-DMINC  .OR.
     &          POS(3,NI).LT.DMINC .OR. POS(3,NI) .GT. NRZ-DMINC ) THEN 
C-----------------------------------------------------------------------
C  truncate charge around ions using a cutoff function
C-----------------------------------------------------------------------
              IF (ISET.EQ.ISET0)
     &           WRITE(*,'(I5,3F5.2)')NI,POS(1,NI),POS(2,NI),POS(3,NI)

              DO 400 N3=1,NGZ
                DX=MOD((N3-1)*F3-POS(3,NI)+50.5,FNRX) -.5
              DO 400 N2=1,NGY
                DY=MOD((N2-1)*F2-POS(2,NI)+50.5,FNRY) -.5
              DO 400 N1=1,NGX
                DZ=MOD((N1-1)*F1-POS(1,NI)+50.5,FNRZ) -.5

                D =SQRT((DX*A(1,1)+DY*A(1,2)+DZ*A(1,3))**2 
     &                + (DX*A(2,1)+DY*A(2,2)+DZ*A(2,3))**2
     &                + (DX*A(3,1)+DY*A(3,2)+DZ*A(3,3))**2)
                TRUNC=(DMIN-D)/WI
                C(N1,N2,N3)=C(N1,N2,N3)/(EXP(TRUNC)+1)
 400          CONTINUE
            ENDIF
 300      CONTINUE
        ENDIF 

C--------------------------------------------------------------------
C  write charge-density
C--------------------------------------------------------------------
        WRITE(*,*)'writing charge now'
        NBOUND=0
        NGZMAX=NGZ
        NGZMIN=0

        NGXW=(NGX-NBOUND*2)/ISKIP
        NGYW=(NGY-NBOUND*2)/ISKIP
        NGZW=(NGZMAX-NGZMIN-NBOUND*2)/ISKIP

        WRITE(20,1) ISET,NGXW*NGYW*NGZW
    1 FORMAT ('object "',I3,
     &        'w" class array items ',I8,' data follows')

C 10F8.1 -> 10F6.1 in der naechsten Zeile                       
        WRITE(20,'(10F8.3)') (((C(N1,N2,N3)
     &               ,N3=1+NGZMIN+NBOUND,NGZMAX-NBOUND,ISKIP)     
     &               ,N2=1+NBOUND,NGY-NBOUND,ISKIP)
     &               ,N1=1+NBOUND,NGX-NBOUND,ISKIP)

        WRITE(20,2) ISET,NGXW ,NGYW ,NGZW,
C     &    (A(I,1)/NGX*NBOUND,I=1,3),
     &    0,0,A(3,3)/NGZ*NGZMIN,
     &    (A(I,1)/NGX*NRX*ISKIP,I=1,3),
     &    (A(I,2)/NGY*NRY*ISKIP,I=1,3),
     &    (A(I,3)/NGZ*NRZ*ISKIP,I=1,3),
     &    ISET,NGXW,NGYW,NGZW,    
     &    ISET,ISET,ISET,ISET

    2 FORMAT(
     & 'attribute "dep" string "positions"'/
     & / 
     & 'object "',I3,'g" class gridpositions counts',3I4/
     & ' origin   ',3F7.3, /
     & ' delta    ',3F6.3, /
     & ' delta    ',3F6.3, /
     & ' delta    ',3F6.3, /
     & /
     & 'object "',I3,'c" class gridconnections counts',3I4/
     & ' attribute "element type" string "cubes"' /
     & ' attribute "ref" string "positions"' /
     & /
     & 'object "',I3,'f" class field' /
     & ' component "data" "',I3,'w"' /
     & ' component "positions" "',I3,'g"' /
     & ' component "connections" "',I3,'c"'//)
      ENDIF

C=======================================================================
C  create the file which contains the positions (pos.dx)
C=======================================================================
      IF (LEIN) THEN
        WRITE(21,51) ISET,NIONS+8
      ELSE
        WRITE(21,51) ISET,NIONS
      ENDIF

   51 FORMAT ('object "',I3,
     & 'p" class array type float rank 1 shape 3 items '
     & ,I4,' data follows')
      DO 310 NI=1,NIONS
        WRITE(21,'(3F10.3)') 
     &    POS(1,NI)*A(1,1)+POS(2,NI)*A(1,2)+POS(3,NI)*A(1,3),
     &    POS(1,NI)*A(2,1)+POS(2,NI)*A(2,2)+POS(3,NI)*A(2,3),
     &    POS(1,NI)*A(3,1)+POS(2,NI)*A(3,2)+POS(3,NI)*A(3,3)
 310  CONTINUE
      IF (LEIN) THEN
      DO IX=0,1
       DO IY=0,1
        DO IZ=0,1
         WRITE(21,'(3F10.3)')
     &    IX*A(1,1)+IY*A(1,2)+IZ*A(1,3),
     &    IX*A(2,1)+IY*A(2,2)+IZ*A(2,3),
     &    IX*A(3,1)+IY*A(3,2)+IZ*A(3,3)
        ENDDO
       ENDDO
      ENDDO
      ENDIF

      IF (LEIN) THEN
        WRITE(21,52) ISET,NIONS+8
      ELSE
        WRITE(21,52) ISET,NIONS
      ENDIF

  52  FORMAT('object "',I3,
     &  'd" class array type float rank 0 items ',
     &  I4,' data follows')

      WRITE(21,'(1F8.3)') (ANEIG(I),I=1,NIONS)
      IF (LEIN) THEN
        WRITE(21,'(1F8.3)') (5.0,I=1,8)
      ENDIF

      WRITE(21,53) 
  53  FORMAT(' attribute "dep" string "positions"')

C--------------------------------------------------------------------
C   connection 
C--------------------------------------------------------------------
      NCON=0
      DO 900 NI=1,NIONS
        NCON=NCON+NEIGN(NI)  
  900 CONTINUE
      WRITE(*,*)'Anzahl der Verbindungen',NCON/2

      IF (LEIN) THEN
        WRITE(21,54) ISET,NCON/2+12
      ELSE
        WRITE(21,54) ISET,NCON/2
      ENDIF

  54  FORMAT('object "',I3,
     &  'c" class array type int rank 1 shape 2 items ',
     &  I4,' data follows')

      DO 910 NI=1,NIONS
      DO 910 I=1,NEIGN(NI)
        NNI=NEIGT(NI,I)  
        IF (NI.LT.NNI) WRITE(21,'(2I4)') NI-1,NNI-1
  910 CONTINUE
      IF (LEIN) THEN
        WRITE(21,'(2I4)') NIONS+0,NIONS+1
        WRITE(21,'(2I4)') NIONS+0,NIONS+2
        WRITE(21,'(2I4)') NIONS+0,NIONS+4
        WRITE(21,'(2I4)') NIONS+1,NIONS+3
        WRITE(21,'(2I4)') NIONS+1,NIONS+5
        WRITE(21,'(2I4)') NIONS+2,NIONS+3
        WRITE(21,'(2I4)') NIONS+2,NIONS+6
        WRITE(21,'(2I4)') NIONS+3,NIONS+7
        WRITE(21,'(2I4)') NIONS+4,NIONS+5
        WRITE(21,'(2I4)') NIONS+4,NIONS+6
        WRITE(21,'(2I4)') NIONS+5,NIONS+7
        WRITE(21,'(2I4)') NIONS+6,NIONS+7
      ENDIF

      WRITE(21,55) 
  55  FORMAT(
     & ' attribute "element type" string "lines"'/
     & ' attribute "ref" string "positions"')

C--------------------------------------------------------------------
C  color and opacity 
C--------------------------------------------------------------------
      IF (LEIN) THEN
        WRITE(21,56) ISET,NIONS+8
      ELSE
        WRITE(21,56) ISET,NIONS
      ENDIF

   56 FORMAT ('object "',I3, 
     & 'co" class array type float rank 1 shape 3 items '
     & ,I4,' data follows')

      WRITE(21,'(3F6.3)') (1.0,0.0,0.0,I=1,NIONS)
      IF (LEIN) THEN
        WRITE(21,'(3F6.3)') (0.0,0.0,0.0,I=1,8)
      ENDIF

      WRITE(21,53)

      IF (LEIN) THEN
        WRITE(21,57) ISET,NIONS+8
      ELSE
        WRITE(21,57) ISET,NIONS
      ENDIF

   57 FORMAT ('object "',I3, 
     & 'o" class array type float rank 1 shape 1 items '
     & ,I4,' data follows')

      DO 915 NI=1,NIONS
        AB=MIN(ABS(MOD(POS(1,NI)+50.5,FNRZ)-.5),
     &         ABS(MOD(POS(2,NI)+50.5,FNRZ)-.5),
     &         ABS(MOD(POS(3,NI)+50.5,FNRZ)-.5))
        IF (AB*SCALE.GT.DMINE.OR.DMINE.EQ.0) THEN 
          WRITE(21,'(3F6.3)') 1.0 
        ELSE
          WRITE(21,'(3F6.3)') AB*SCALE/DMINE 
        ENDIF 
  915 CONTINUE
      IF (LEIN) THEN
        DO NI=1,8
          WRITE(21,'(3F6.3)') .2
        ENDDO
      ENDIF

      WRITE(21,53)

C--------------------------------------------------------------------
C  data2 depends on connections
C--------------------------------------------------------------------
      IF (LEIN) THEN
        WRITE(21,58) ISET,NCON/2+12
      ELSE
        WRITE(21,58) ISET,NCON/2
      ENDIF

  58  FORMAT('object "',I3,
     &  'd2" class array type float rank 0 items ',
     &  I4,' data follows')

      DO 930 NI=1,NIONS
      DO 930 I=1,NEIGN(NI)
        NII=NEIGT(NI,I)  
        IF (NI.LT.NII) THEN
          AB=MIN(ABS(MOD(POS(1,NI)+50.5,FNRX)-.5),
     &           ABS(MOD(POS(2,NI)+50.5,FNRY)-.5),
     &           ABS(MOD(POS(3,NI)+50.5,FNRZ)-.5),
     &           ABS(MOD(POS(1,NII)+50.5,FNRX)-.5),
     &           ABS(MOD(POS(2,NII)+50.5,FNRY)-.5),
     &           ABS(MOD(POS(3,NII)+50.5,FNRZ)-.5))
          IF (AB*SCALE.GT.DMINE.OR.DMINE.EQ.0) THEN 
            AB=1.0
          ELSE
            AB=AB*SCALE/DMINE 
          ENDIF 

          DX = MOD(POS(1,NI)-POS(1,NII)+50.5,FNRX) -.5
          DY = MOD(POS(2,NI)-POS(2,NII)+50.5,FNRY) -.5
          DZ = MOD(POS(3,NI)-POS(3,NII)+50.5,FNRZ) -.5
          D  = SQRT((DX*A(1,1)+DY*A(1,2)+DZ*A(1,3))**2
     &            + (DX*A(2,1)+DY*A(2,2)+DZ*A(2,3))**2
     &            + (DX*A(3,1)+DY*A(3,2)+DZ*A(3,3))**2)
          IF (D.LT.DNEIG1) THEN 
            AKOOR=1
          ELSE IF (D.LT.DNEIG2) THEN
            AKOOR=(DNEIG2-D)/(DNEIG2-DNEIG1)
          ELSE
            AKOOR=-1
          ENDIF
          WRITE(21,'(3F6.3)') AKOOR*AB
        ENDIF
  930 CONTINUE
      IF (LEIN) THEN
        DO NI=1,12
          WRITE(21,'(3F6.3)') .2
        ENDDO
      ENDIF

      WRITE(21,59) 
  59  FORMAT(
     & ' attribute "dep" string "connections"')

C-----------------------------------------------------------------------
C  description of field 
C-----------------------------------------------------------------------
      WRITE(21,60) ISET,ISET,ISET,ISET,ISET,ISET,ISET
  60  FORMAT(
     & 'object "',I3,'f" class field'/
     & '  component "positions" value "',I3,'p"'/
     & '  component "data" value "',I3,'d"'/
     & '  component "data2" value "',I3,'d2"'/
     & '  component "connections" value "',I3,'c"'/
     & '  component "opacities" value "',I3,'o"'/
     & '  component "colors" value "',I3,'co"'/)
             
      ISET=ISET+1
      GOTO 1300

1100  WRITE(*,*)'Error, while reading CHG at',N1,N2,N3
2000  CONTINUE    

C-----------------------------------------------------------------------
C  write time-series description field 
C-----------------------------------------------------------------------
      WRITE(21,71) 
      WRITE(20,71) 
   71 FORMAT ('object 1 class series')
      
      DO 920 I=ISET0,ISET-1
        WRITE(20,72) I-ISET0,I,I
 920    WRITE(21,73) I-ISET0,I,I

  72  FORMAT(
     & 'member ',I3,' value "',I3,'f" position ',I3)
  73  FORMAT(
     & 'member ',I3,' value "',I3,'f" position ',I3)

      END

***************** SUBROUTINE READAT  ***********************************
*
* read atomic charge-density
*
************************************************************************

      SUBROUTINE READAT(NPSPTS,NTYPD,NTYP,ZVAL,PSGMAX,PSPRHO)
      CHARACTER*40 SZNAM
      CHARACTER*1  CSEL 
      DIMENSION PSPRHO(NPSPTS,NTYPD),ZVAL(NTYPD),PSGMAX(NTYPD)

      NTYP=0
      OPEN(UNIT=10,FILE='POTCAR',STATUS='OLD',ERR=100)
      WRITE(*,*)'till now I only understand the old POTCAR-format ;-((('
      WRITE(*,*)'so if you need the POTCAR file - for instance'//
     &          'to subtract the atomic charge'
      WRITE(*,*)'you have to rewrite the subroutine READAT !!!'
      GOTO 100

      READ(10,'(A40)',END=100,ERR=100) SZNAM
      NTYP=NTYP+1

      WRITE(*,*)'POTCAR:  ',SZNAM

      READ(10,*) ZVAL(NTYP)
      WRITE(*,'(10H  Valenz:  ,F6.3)')ZVAL(NTYP)

      READ(10,*)
      READ(10,*) PSGMAX(NTYP)

      READ(10,*) (DUMMY,I=1,NPSPTS)
      WRITE(*,*)' local pseudopotential read in'
      READ(10,'(1X,A1)') CSEL
      IF (CSEL.EQ.'g') THEN
        READ(10,*) LEXCHG
        READ(10,'(1X,A1)') CSEL
      ELSE
        LEXCHG=0
      ENDIF

      IF (CSEL.EQ.'c') THEN
        WRITE(*,*)' partial core-charges read in'
        READ(10,*) (DUMMY,I=1,NPSPTS)
        READ(10,*)
      ENDIF

      READ(10,*) (PSPRHO (I,NTYP),I=1,NPSPTS)
      WRITE(*,*)' atomic valenz-charges read in'


 100  WRITE(*,*)NTYP,' pseudopotentials read in'
      CLOSE(10)
      RETURN
      END
     
C--------------------------------------------------------------------
C  old File Format
C--------------------------------------------------------------------
C      READ(10,'(A40)') SZ2
C      SCALE=11.343
C
C      A(1,1)=SCALE
C      A(2,1)=0
C      A(3,1)=0
C      A(1,2)=0
C      A(2,2)=SCALE
C      A(3,2)=0
C      A(1,3)=0
C      A(2,3)=0
C      A(3,3)=SCALE
C
C      NIONS=64
C
C      DO 100 K=1,NIONS
C        READ(10,*) POS(1,K),POS(2,K),POS(3,K)
C  100 CONTINUE
C      ENDIF

C-----old format - charge density
C      READ(10,'(10F6.1)')
C     &  (((C(N1,N2,N3),N1=1,NGX),N2=1,NGY),N3=1,NGZ)

